unit AlphaBlendingMain;

interface

uses
  SysUtils, Forms,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    MultiMap: TxorDXMultiMap;
    Polygon: TxorDXPolygon;
    AlphaBlending: TxorDXAlphaBlending;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Graphics;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  //creation
  Scene  := TxorDXScene.Create(Handle);
  Camera := TxorDXCamera.Create(Scene);
  MultiMap := TxorDXMultiMap.Create(Scene, 50, 50);
  Polygon := TxorDXPolygon.Create(Scene, 18);
  AlphaBlending := TxorDXAlphaBlending.Create(Scene);
  Scene.CreateBuffers;
  //start
  AlphaBlending.BlendSrcAlpha;
  AlphaBlending.Activate;
  Polygon.DrawMode := xorDXDrawTriangleList;
  //using alpha 0.7 will result in 30% transparency
  PxorDXVertexClassic(Polygon.Vertex[0])^ := GetVertexClassic(0,0,0, 0,0,0, 0,0,255, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[1])^ := GetVertexClassic(1,0,0, 0,0,0, 0,255,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[2])^ := GetVertexClassic(1,1,0, 0,0,0, 255,0,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[3])^ := GetVertexClassic(0,0,0, 0,0,0, 0,0,255, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[4])^ := GetVertexClassic(0,1,0, 0,0,0, 0,255,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[5])^ := GetVertexClassic(1,1,0, 0,0,0, 255,0,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[6])^ := GetVertexClassic(1,1,1, 0,0,0, 0,0,255, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[7])^ := GetVertexClassic(1,0,0, 0,0,0, 0,255,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[8])^ := GetVertexClassic(1,1,0, 0,0,0, 255,0,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[9])^ := GetVertexClassic(1,1,1, 0,0,0, 0,0,255, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[10])^ := GetVertexClassic(1,0,0, 0,0,0, 0,255,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[11])^ := GetVertexClassic(1,0,1, 0,0,0, 255,0,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[12])^ := GetVertexClassic(1,1,1, 0,0,0, 0,0,255, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[13])^ := GetVertexClassic(0,1,0, 0,0,0, 0,255,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[14])^ := GetVertexClassic(1,1,0, 0,0,0, 255,0,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[15])^ := GetVertexClassic(1,1,1, 0,0,0, 0,0,255, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[16])^ := GetVertexClassic(0,1,0, 0,0,0, 0,255,0, 0.7);
  PxorDXVertexClassic(Polygon.Vertex[17])^ := GetVertexClassic(0,1,1, 0,0,0, 255,0,0, 0.7);
  MultiMap.MapTorus(GetVector(0.5,0,2), 0.3, 1.3);
  MultiMap.MonoChrom(clGreen, 1.0); //setting alpha to 1.0 result in no transparency
  Camera.SetStandard;
  Camera.MoveForward(2, xorDXCamID);
  Camera.Activate;
  Scene.StartLooping(Draw);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(AlphaBlending) then FreeAndNil(AlphaBlending);
  if Assigned(Polygon) then FreeAndNil(Polygon);
  if Assigned(MultiMap) then FreeAndNil(MultiMap);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;
  with MultiMap do
    begin
    Insert;
    Draw;
    end;
  with Polygon do
    begin
    Insert;
    RotateX(-0.01);
    Draw;
    end;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
