unit BowlingMain;

interface

uses
  SysUtils, Forms,
  xorDX8, Classes, Controls, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    procedure DrawMultiMaps;
    procedure GetKeyboard(var MoveZ: Single);
  public
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Keyboard: TxorDXKeyboard;
    MultiMap: TxorDXMultiMap;
    Ball: TxorDXBall;
    Grid: TxorDXGridMap;
    Texture: TxorDXTexture;
    Sun, Sun2: TxorDXDirectionalLight;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Windows;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  i, j: Integer;
begin
  Randomize;

  //creation
  Scene  := TxorDXScene.Create(Handle);

  Camera := TxorDXCamera.Create(Scene);
  Camera.Position := GetVector(0, 0.5, -3);
  Camera.Ratio := Width / Height;
  Camera.Activate;
  Camera.FixTransformation;

  Keyboard := TxorDXKeyboard.Create(Scene);

  MultiMap := TxorDXMultiMap.CreateParam(Scene, 40, 40, xorDXVertexSizeTexture, xorDXVertexFlagTexture); //creation with texture coordinates

  Ball := TxorDXBall.Create(Scene, 40, 40, 1);  //ball with rad=1 and middle = 000
  Ball.MaterialSpecularColor := $00FFFFFF;
  Ball.MaterialEmissiveColor := $00000000;
  Ball.FixTransformation;

  Texture := TxorDXTexture.Create(Scene);
  Texture.LoadFromFile('..\dali.bmp', False, $00FFFFFF, 0);
  Texture.Activate;

  Grid := TxorDXGridMap.Create(Scene, 10, 20);
  with Grid do
    for i := 0 to MapWidth - 1 do
      for j := 0 to MapHeight - 1 do
        PxorDXVertexClassic(Vertex[i,j])^ := GetVertexClassic(6 * i/MapWidth - 3, -1.5 + Random/4, 12 * j/MapHeight - 2, 0,0,0, 0,0,0, 0);
  Grid.InitNormalVectors;

  Sun := TxorDXDirectionalLight.Create(Scene);
  Sun.Direction := GetVector(0, -10, 0);
  Sun.Activate;

  Sun2 := TxorDXDirectionalLight.Create(Scene);
  Sun2.Direction := GetVector(0, 0, 1);
  Sun2.DiffuseColor := $00FFFFFF;
  Sun2.SpecularColor := $00808080;
  Sun2.Activate;

  Scene.CreateBuffers;
  Scene.StartLooping(Draw);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(Sun2) then FreeAndNil(Sun2);
  if Assigned(Sun) then FreeAndNil(Sun);
  if Assigned(Grid) then FreeAndNil(Grid);
  if Assigned(Texture) then FreeAndNil(Texture);
  if Assigned(Ball) then FreeAndNil(Ball);
  if Assigned(MultiMap) then FreeAndNil(MultiMap);
  if Assigned(Keyboard) then FreeAndNil(Keyboard);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
const
  MoveZ: Single = 0.01;
begin
  Scene.Clear;

  GetKeyboard(MoveZ);

  //Move Ball and Camera forward/backward
  Ball.Move(0, 0, MoveZ);
  Camera.MoveForward(MoveZ, xorDXCamHuman);
  Camera.Activate;

  Grid.Insert;
  Grid.Draw;

  DrawMultiMaps;

  Ball.Insert;
  Ball.Draw;

end;

procedure TForm1.GetKeyboard(var MoveZ: Single);
const
  MoveX = 0.03;
begin
  with Keyboard do
    if Get and KeyPressedAny then
      begin
      if KeyPressed(VK_ESCAPE) then
        begin
        Ball.UndoTransformation;
        Camera.UndoTransformation;
        end;
      if KeyPressed(VK_LEFT) then //move both left
        begin
        Ball.Move(-MoveX, 0, 0);
        Camera.MoveHoriz(-MoveX, xorDXCamHuman);
        end;
      if KeyPressed(VK_RIGHT) then //move both right
        begin
        Ball.Move(MoveX, 0, 0);
        Camera.MoveHoriz(MoveX, xorDXCamHuman);
        end;
      if KeyPressed(VK_UP) then MoveZ := Abs(MoveZ);
      if KeyPressed(VK_DOWN) then MoveZ := -Abs(MoveZ);
      end;
end;

procedure TForm1.DrawMultiMaps;
begin
  with MultiMap do
    begin
    Texture.ColorStageBlendTextureAndDiffuseColor(bop1X);

    MonoChrom($00FFFFFF);
    MaterialSpecularColor := 0;
    MaterialEmissiveColor := $00909090;
    MapCone(GetVector(0,2,0), GetVector(0,0,0), 0.4);
    Insert;

    RotateX(1.2);
    RotateY(2.2);
    Move(0.35, -1, 5);
    Draw;
    UndoTransformation;

    RotateX(2.9);
    RotateY(2.1);
    Move(0.4, 3.3, 3.5);
    Draw;
    UndoTransformation;

    RotateX(1.1);
    RotateY(0.6);
    Move(0.3, 1.0, 1.1);
    Draw;
    UndoTransformation;

    RotateX(1.9);
    RotateY(-0.2);
    Move(2.3, 0.5, 2.4);
    Draw;
    UndoTransformation;

    RotateX(0.2);
    RotateY(0.7);
    Move(2.8, 0, 0.7);
    Draw;
    UndoTransformation;

    RotateX(1.6);
    RotateY(0.7);
    Move(2.9, 0.2, -0.4);
    Draw;
    UndoTransformation;

    RotateX(1.2);
    RotateY(-0.9);
    Move(-0.4, 0, 0);
    Draw;
    UndoTransformation;

    RotateX(0.4);
    RotateY(-2.0);
    Move(-0.1, 0.1, -1.5);
    Draw;
    UndoTransformation;

    RotateX(3.0);
    RotateY(4.2);
    Move(3.1, 1.4, -2);
    Draw;
    UndoTransformation;

    Texture.ColorStageBlendDisable;
    end;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
