unit IndexedPolygonMain;

interface

uses
  SysUtils, Forms,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    IndexedPolygon: TxorDXIndexedPolygon;
    Mouse: TxorDXMouse;
    Keyboard: TxorDXKeyboard;
    procedure Draw(Sender: TObject);
    procedure FillIn;
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Graphics, DirectXGraphics, Windows;

const
  VertexCount = 11; //vertex 0 is unused here
  TriangleCount = 24;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  Scene  := TxorDXScene.Create(Handle);

  Camera := TxorDXCamera.Create(Scene);
  Camera.Position := GetVector(2, 6, -6); //x=right, y=up, z=forth (or back because negativ)
  Camera.LookAt := GetVector(0, 0, 0);
  Camera.Activate;

  Mouse    := TxorDXMouse.Create(Scene);
  Mouse.InvertX := True;
  Keyboard := TxorDXKeyboard.Create(Scene);

  {VertexCount fixes the size of the vertex-array;
   xorDXDrawTriangleList that the first three indices form a triangle, the second
   three indices also and so on
   GetEntryCount gets the size of the triangle index array, here triangleCount*3
  }
  IndexedPolygon := TxorDXIndexedPolygon.Create(Scene, VertexCount,
                            GetEntryCount(xorDXDrawTriangleList, TriangleCount));
  FillIn; //fill polygon

  Scene.CreateBuffers;
  Scene.StartLooping(Draw); //loop drawing routine
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(IndexedPolygon) then FreeAndNil(IndexedPolygon);
  if Assigned(Keyboard) then FreeAndNil(Keyboard);
  if Assigned(Mouse) then FreeAndNil(Mouse);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;

  if Mouse.Get then //read mouse buffer
    with IndexedPolygon do
      begin
      MoveCenter(True);
      RotateY(Mouse.DX/100);
      RotateX(Mouse.DY/100);
      MoveCenter(False);
      if Mouse.ClickedLeft then UndoTransformation; //undo not fixed transformations
      end;

  if Keyboard.Get and Keyboard.KeyPressedAny then //read keyboard buffer
    with Camera do
      begin
      //use windows "virtual keycodes"
      if Keyboard.KeyPressed(VK_ESCAPE) then Application.Terminate;
      if Keyboard.KeyPressed(VK_LEFT) then MoveHoriz(-0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_RIGHT) then MoveHoriz(0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_UP) then MoveForward(0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_DOWN) then MoveForward(-0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_PRIOR) then MoveVert(0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_NEXT) then MoveVert(-0.03, xorDXCamId);
      Activate;
      end;

  IndexedPolygon.Insert;
  IndexedPolygon.Draw;
end;

procedure TForm1.FillIn;
begin
  with IndexedPolygon do
    begin
    DrawMode := xorDXDrawTriangleList;
    PxorDXVertexClassic(Vertex[0])^ := GetVertexClassic( 0,0,0,  0,0,0, clBlack, 0); //unused
    PxorDXVertexClassic(Vertex[1])^ := GetVertexClassic( 0,0,0,  0,0,0, clBlack, 0);
    PxorDXVertexClassic(Vertex[2])^ := GetVertexClassic( 0,0,3,  0,0,0, clBlue, 0);
    PxorDXVertexClassic(Vertex[3])^ := GetVertexClassic( 0,1,0,  0,0,0, clRed, 0);
    PxorDXVertexClassic(Vertex[4])^ := GetVertexClassic( 4,0,1,  0,0,0, clGreen, 0);
    PxorDXVertexClassic(Vertex[5])^ := GetVertexClassic( 2,2,4,  0,0,0, clYellow, 0);
    PxorDXVertexClassic(Vertex[6])^ := GetVertexClassic( 1,4,0,  0,0,0, clLime, 0);
    PxorDXVertexClassic(Vertex[7])^ := GetVertexClassic( 3,1,1,  0,0,0, clFuchsia, 0);
    PxorDXVertexClassic(Vertex[8])^ := GetVertexClassic( 3,2,3,  0,0,0, clGray, 0);
    PxorDXVertexClassic(Vertex[9])^ := GetVertexClassic( 2,4,1,  0,0,0, clTeal, 0);
    PxorDXVertexClassic(Vertex[10])^ := GetVertexClassic( 4,3,3,  0,0,0, clNavy, 0);
    Triangle[0]^ := GetTriangle(1, 2, 3);
    Triangle[1]^ := GetTriangle(1, 2, 4);
    Triangle[2]^ := GetTriangle(1, 3, 5);
    Triangle[3]^ := GetTriangle(1, 4, 6);
    Triangle[4]^ := GetTriangle(1, 5, 7);
    Triangle[5]^ := GetTriangle(1, 6, 8);
    Triangle[6]^ := GetTriangle(1, 7, 9);
    Triangle[7]^ := GetTriangle(1, 8, 10);
    Triangle[8]^ := GetTriangle(1, 9, 10);
    Triangle[9]^ := GetTriangle(2, 3, 6);
    Triangle[10]^ := GetTriangle(2, 4, 8);
    Triangle[11]^ := GetTriangle(2, 5, 7);
    Triangle[12]^ := GetTriangle(2, 5, 9);
    Triangle[13]^ := GetTriangle(2, 6, 9);
    Triangle[14]^ := GetTriangle(2, 7, 8);
    Triangle[15]^ := GetTriangle(3, 5, 8);
    Triangle[16]^ := GetTriangle(3, 6, 8);
    Triangle[17]^ := GetTriangle(4, 5, 8);
    Triangle[18]^ := GetTriangle(4, 5, 10);
    Triangle[19]^ := GetTriangle(4, 6, 9);
    Triangle[20]^ := GetTriangle(4, 7, 9);
    Triangle[21]^ := GetTriangle(4, 7, 10);
    Triangle[22]^ := GetTriangle(5, 9, 10);
    Triangle[23]^ := GetTriangle(7, 8, 10);
    InitNormalVectors;
    end;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
