unit KeyAndMouseMain;

interface

uses
  SysUtils, Forms, Dialogs, Windows, 
  xorDX8, Classes, Controls, StdCtrls;

type
  TForm1 = class(TForm)
    CheckBox1: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
  private
    { Private-Deklarationen }
    CamTrans: TxorDXCameraTransformation; //kind of camera transformation
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Rect: TxorDXRect;
    Mouse: TxorDXMouse;
    Keyboard: TxorDXKeyboard;
    Font: TxorDXFont;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  //creation
  Scene    := TxorDXScene.Create(Handle);
  Camera   := TxorDXCamera.Create(Scene);
  Rect     := TxorDXRect.Create(Scene, 30, 15, 4, 2); //set standard coordinates and colors
  Mouse    := TxorDXMouse.Create(Scene);
  Keyboard := TxorDXKeyboard.Create(Scene);
  Font     := TxorDXFont.Create(Scene);
  Scene.CreateBuffers;
  //start
  CamTrans := xorDXCamHuman;
  Camera.SetStandard;
  Camera.Ratio := Width/Height;
  Camera.Activate;
  Mouse.Speed := 1/600;
  Scene.StartLooping(Draw); //loop drawing routine
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(Font) then FreeAndNil(Font);
  if Assigned(Keyboard) then FreeAndNil(Keyboard);
  if Assigned(Mouse) then FreeAndNil(Mouse);
  if Assigned(Rect) then FreeAndNil(Rect);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
const
  CamTransStr: Array[0..3] of String[5] = ('Human','ID','Scene','Up');
begin
  Scene.Clear;

  if Mouse.Get then //read mouse buffer
    with Camera do
      begin
      RotateHoriz(Mouse.DX, CamTrans);
      RotateVert(-Mouse.DY, CamTrans);
      if Mouse.ClickedLeft then UndoTransformation;
      //coordinates
      Font.TextOut(Classes.Rect(0,0, 400, 90), xorDXFontBottom, $00000000, 1.0, 'Real Screen Coordinates:');
      Font.TextOut(Classes.Rect(0,0, 400,130), xorDXFontBottom, $00000000, 1.0, 'ScreenX = ' + IntToStr(Mouse.ScreenX));
      Font.TextOut(Classes.Rect(0,0, 400,170), xorDXFontBottom, $00000000, 1.0, 'ScreenY = ' + IntToStr(Mouse.ScreenY));
      Font.TextOut(Classes.Rect(0,0, 400,210), xorDXFontBottom, $00000000, 1.0, 'ScreenWheel = ' + IntToStr(Mouse.ScreenWheel));
      Font.TextOut(Classes.Rect(0,0, 400,250), xorDXFontBottom, $00000000, 1.0, 'ScreenDX = ' + IntToStr(Mouse.ScreenDX));
      Font.TextOut(Classes.Rect(0,0, 400,290), xorDXFontBottom, $00000000, 1.0, 'ScreenDY = ' + IntToStr(Mouse.ScreenDY));
      Font.TextOut(Classes.Rect(0,0, 400,330), xorDXFontBottom, $00000000, 1.0, 'ScreenDWheel = ' + IntToStr(Mouse.ScreenDWheel));
      Font.TextOut(Classes.Rect(0,0, 400,400), xorDXFontBottom, $00000000, 1.0, 'Imaginary Screen Coordinates:');
      Font.TextOut(Classes.Rect(0,0, 400,440), xorDXFontBottom, $00000000, 1.0, 'X = ' + FloatToStr(Mouse.X));
      Font.TextOut(Classes.Rect(0,0, 400,480), xorDXFontBottom, $00000000, 1.0, 'Y = ' + FloatToStr(Mouse.Y));
      Font.TextOut(Classes.Rect(0,0, 400,520), xorDXFontBottom, $00000000, 1.0, 'Wheel = ' + FloatToStr(Mouse.Wheel));
      Font.TextOut(Classes.Rect(0,0, 400,560), xorDXFontBottom, $00000000, 1.0, 'DX = ' + FloatToStr(Mouse.DX));
      Font.TextOut(Classes.Rect(0,0, 400,600), xorDXFontBottom, $00000000, 1.0, 'DY = ' + FloatToStr(Mouse.DY));
      Font.TextOut(Classes.Rect(0,0, 400,640), xorDXFontBottom, $00000000, 1.0, 'DWheel = ' + FloatToStr(Mouse.DWheel));
      Font.TextOut(Classes.Rect(0,0, 400,680), xorDXFontBottom, $00000000, 1.0, 'Speed = ' + FloatToStr(Mouse.Speed));
      end;

  with Keyboard, Camera do
    if Get and KeyPressedAny then //read keyboard buffer
      begin
      //use windows "virtual keycodes"
      if KeyPressed(VK_ESCAPE) then Application.Terminate;
      if KeyPressed(VK_LEFT) then MoveHoriz(-0.03, CamTrans);
      if KeyPressed(VK_RIGHT) then MoveHoriz(0.03, CamTrans);
      if KeyPressed(VK_UP) then MoveForward(0.03, CamTrans);
      if KeyPressed(VK_DOWN) then MoveForward(-0.03, CamTrans);
      if KeyPressed(VK_PRIOR) then MoveVert(0.03, CamTrans);
      if KeyPressed(VK_NEXT) then MoveVert(-0.03, CamTrans);
      if KeyPressed(Ord('1')) then CamTrans := xorDXCamHuman;
      if KeyPressed(Ord('2')) then CamTrans := xorDXCamID;
      if KeyPressed(Ord('3')) then CamTrans := xorDXCamScene;
      if KeyPressed(Ord('4')) then CamTrans := xorDXCamUp;
      end;

  Camera.Activate; //set camera changings
  Caption := 'xorDX - Example    Camera-Transformation = xorDXCam' + CamTransStr[CamTrans] + '   (press 1..4 to change)          Use CURSOR, PAGE and ESC keys and mouse!';
  Rect.Insert;
  Rect.Draw;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  Mouse.ReturnScreenCoordinates := CheckBox1.Checked;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
