unit MapMain;

interface

uses
  SysUtils, Forms,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Map: TxorDXMap;
    Poly: TxorDXIndexedPolygon;
    Tex0: TxorDXTexture;
    Tex1: TxorDXTexture;
    Spot: TxorDXSpotLight;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Graphics;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  u,v: Integer;
begin

//first step: creation

//  Scene  := TxorDXScene.CreateFullScreen(Handle);   //close with Alt-F4
  Scene  := TxorDXScene.Create(Handle);
  Camera := TxorDXCamera.Create(Scene);
  Spot := TxorDXSpotLight.Create(Scene);
  Spot.Position := GetVector(0.3,7.5,2.8);
  Spot.Direction := GetVector(-0.2,-1,0.2);
  Spot.DiffuseColor := clYellow;
  Spot.Theta := 0.9;
  Spot.Phi := 1.3;
  Spot.Activate;

  //30x30 Map
  Map := TxorDXMap.CreateParam(Scene, 30, 30, xorDXVertexSizeTexture, xorDXVertexFlagTexture);
  for u := 0 to Map.MapWidth - 1 do
    for v := 0 to Map.MapHeight - 1 do
      with PxorDXVertexTexture(Map.Vertex[u,v])^ do
        begin
        Position := GetVector(30 * u / (Map.MapWidth - 1) - 15, Trunc(Random(11)/10) + Random/3 - 1.5, 30 * v / (Map.MapHeight - 1) - 1);
        TexCoordinates := GetCoord(3 * u / (Map.MapWidth - 1), 3 * v / (Map.MapHeight - 1));
        end;
  Map.InitNormalVectors;
  Map.MonoChrom(clWhite);

  //polygon 10 vertices, 10 triangles
  Poly := TxorDXIndexedPolygon.CreateParam(Scene, 10, GetEntryCount(xorDXDrawTriangleList, 10), xorDXVertexSizeTexture, xorDXVertexFlagTexture);
  Poly.DrawMode := xorDXDrawTriangleList;
  PxorDXVertexTexture(Poly.Vertex[0])^ := GetVertexTexture(0.1, 0.0, 0.0, 0,0,0, 0.0, 0.9,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[1])^ := GetVertexTexture(0.9, 0.0, 0.0, 0,0,0, 0.9, 0.9,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[2])^ := GetVertexTexture(0.5, 0.5,-0.3, 0,0,0, 0.5, 0.75, clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[3])^ := GetVertexTexture(0.2, 1.0, 0.0, 0,0,0, 0.2, 0.6,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[4])^ := GetVertexTexture(0.8, 1.0, 0.0, 0,0,0, 0.8, 0.6,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[5])^ := GetVertexTexture(0.1, 1.3, 0.0, 0,0,0, 0.1, 0.5,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[6])^ := GetVertexTexture(0.5, 1.3,-0.3, 0,0,0, 0.4, 0.5,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[7])^ := GetVertexTexture(0.9, 1.3, 0.0, 0,0,0, 0.8, 0.5,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[8])^ := GetVertexTexture(0.2, 2.0, 0.0, 0,0,0, 0.2, 0.1,  clWhite, 0);
  PxorDXVertexTexture(Poly.Vertex[9])^ := GetVertexTexture(0.8, 2.0, 0.0, 0,0,0, 0.8, 0.1,  clWhite, 0);
  Poly.Triangle[0]^ := GetTriangle(0,1,2);
  Poly.Triangle[1]^ := GetTriangle(0,2,3);
  Poly.Triangle[2]^ := GetTriangle(1,2,4);
  Poly.Triangle[3]^ := GetTriangle(2,3,4);
  Poly.Triangle[4]^ := GetTriangle(3,4,6);
  Poly.Triangle[5]^ := GetTriangle(3,5,6);
  Poly.Triangle[6]^ := GetTriangle(4,6,7);
  Poly.Triangle[7]^ := GetTriangle(5,6,8);
  Poly.Triangle[8]^ := GetTriangle(6,7,9);
  Poly.Triangle[9]^ := GetTriangle(6,8,9);
  Poly.InitNormalVectors;
  Poly.MoveCenter(True);
  Poly.FixTransformation;

  Tex0 := TxorDXTexture.Create(Scene);
  Tex0.LoadFromFile('..\Map\psychomap.jpg', 4, False, 0, 0); //4 mip levels
  Tex1 := TxorDXTexture.Create(Scene);
  Tex1.LoadFromFile('..\Map\pitti.jpg');

  Scene.CreateBuffers;

  //start
//  Camera.SetStandard;
  Camera.Activate;
  Scene.StartLooping(Draw); //loop drawing routine
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(Tex1) then FreeAndNil(Tex1);
  if Assigned(Tex0) then FreeAndNil(Tex0);
  if Assigned(Poly) then FreeAndNil(Poly);
  if Assigned(Map) then FreeAndNil(Map);
  if Assigned(Spot) then FreeAndNil(Spot);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;

  Camera.UndoTransformation;
  Camera.MoveForward(Sin(RotationsPerSecond(0.25)), xorDXCamID);
  Camera.Activate;

  //map
  Tex0.Activate;
  Tex0.ColorStageBlendTextureAndDiffuseColor(bop1X);
  Map.Insert;
  Map.Draw;
  Tex0.Deactivate;

  //pitty
  Tex1.Activate;
  Tex1.ColorStageBlendTextureOnly;
  Poly.Insert;
  Poly.RotateY(-0.006);
  Poly.FixTransformation;
  Poly.Move(-1,0,1);
  Poly.Draw;
  Poly.Move(0.5,0,1);
  Poly.Draw;
  Poly.Move(1,0,0);
  Poly.Draw;
  Poly.Move(0.5,0,-1);
  Poly.Draw;
  Poly.UndoTransformation;
  Tex1.Deactivate;

end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

  Randomize;
  RandSeed := 4711;

  InitRotationPerSecond;

end.

