unit MultiMapMain;

interface

uses
  SysUtils, Forms,
  xorDX8, Classes, Controls, StdCtrls, ExtCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    RadioGroup1: TRadioGroup;
    Panel2: TPanel;
    CheckBox1: TCheckBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure RadioGroup1Click(Sender: TObject);
    procedure CheckBox1Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    MultiMap: TxorDXMultiMap;
    Texture: TxorDXTexture;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  //creation
  Scene := TxorDXScene.Create(Handle);
  Camera := TxorDXCamera.Create(Scene);
  MultiMap := TxorDXMultiMap.CreateParam(Scene, 40, 40, xorDXVertexSizeTexture, xorDXVertexFlagTexture); //creation with texture coordinates
  Texture := TxorDXTexture.Create(Scene);
  Scene.CreateBuffers;

  //start
  RadioGroup1Click(nil); //set mapping
  Texture.LoadFromFile('..\dali.bmp', False, $00FFFFFF, 0);
  Texture.ColorStageBlendTextureOnly;
  Texture.Activate;
  Camera.Position := GetVector(0,0,-6);
  Camera.Activate;
  Scene.OutputHandle := Panel2.Handle;
  Scene.StartLooping(Draw); //loop drawing routine
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(Texture) then FreeAndNil(Texture);
  if Assigned(MultiMap) then FreeAndNil(MultiMap);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;
  MultiMap.Insert;
  MultiMap.RotateX(-0.06);
  MultiMap.Draw;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  Camera.Ratio := Panel2.Width / Panel2.Height;
  Camera.Activate;
end;

procedure TForm1.RadioGroup1Click(Sender: TObject);
begin
  Case RadioGroup1.ItemIndex of
    0: MultiMap.MapBall(GetVector(0,2.5,0), 1.5);
    1: MultiMap.MapCone(GetVector(-4,0,0), GetVector(0,3.5,0), 1.5);
    2: MultiMap.MapCylinder(GetVector(-4,1,0), GetVector(0,5.0,0), 0.7);
    3: MultiMap.MapParallelogram(GetVector(-3,2,0), GetVector(-6,-2,0), GetVector(0,-2,0));
    4: MultiMap.MapTorus(GetVector(-1,1,0), 1, 2);
  end;
end;

procedure TForm1.CheckBox1Click(Sender: TObject);
begin
  Texture.IsActive := CheckBox1.Checked;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
