unit MultipassTextureBlendingMain;

interface

uses
  SysUtils, Forms,
  xorDX8, Classes, Controls, StdCtrls, ExtCtrls, ComCtrls;

type
  TForm1 = class(TForm)
    Panel1: TPanel;
    RadioGroup1: TRadioGroup;
    TrackBar1: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
    procedure SetParameters(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    MultiMap: TxorDXMultiMap;
    Texture: TxorDXTexture;
    AlphaBlending: TxorDXAlphaBlending;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Graphics;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  //creation
  Scene    := TxorDXScene.Create(Handle);
  Camera   := TxorDXCamera.Create(Scene);
  MultiMap := TxorDXMultiMap.CreateParam(Scene, 50, 50, xorDXVertexSizeTexture, xorDXVertexFlagTexture);
  Texture := TxorDXTexture.Create(Scene);
  AlphaBlending := TxorDXAlphaBlending.Create(Scene);
  Scene.CreateBuffers;
  //settings
  MultiMap.MapParallelogram(GetVector(-1.5,-1.5,0), GetVector(-1.5,1.5,0), GetVector(1.5,1.5,0));
  MultiMap.MonoAlpha(0.5);                  //set vertices alpha values
  AlphaBlending.BlendSrcAlpha;              //uses alpha of source as blend factor
  Camera.MoveForward(-1, xorDXCamID);
  Camera.MoveVert(-0.4, xordXCamID);
  Camera.Ratio := Width/Height;
  Camera.Activate;
  Scene.OnDraw := Draw;                     //set draw routine
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(AlphaBlending) then FreeAndNil(AlphaBlending);
  if Assigned(Texture) then FreeAndNil(Texture);
  if Assigned(MultiMap) then FreeAndNil(MultiMap);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//onPaint event of TForm1
procedure TForm1.FormPaint(Sender: TObject);
begin
  Scene.Draw;
end;

//user selected parameters
procedure TForm1.SetParameters(Sender: TObject);
begin
  Case RadioGroup1.ItemIndex of
    0: AlphaBlending.BlendSrcAlpha;
    1: AlphaBlending.BlendSrcColor;
    2: AlphaBlending.BlendDestColor;
    3: AlphaBlending.BlendSrcOnly;
    4: AlphaBlending.BlendDestOnly;
  end;
  MultiMap.MonoAlpha(TrackBar1.Position/10);
  Scene.Draw;
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;
  MultiMap.Insert;

  //set first texture on stage 0 (stage blending = how to mix texture and vertex colors and alphas)
  Texture.LoadFromFile('..\dali.bmp');
  Texture.Activate;                         //standard stage 0
  Texture.ColorStageBlendTextureOnly;       //use texture color only, ignore vertex diffuse color
  //first rendering ("destination")
  MultiMap.Draw;                            //first rendering
  Texture.Deactivate;

  //set second texture on stage 0 (stage blending = how to mix texture and vertex colors and alphas)
  Texture.LoadFromFile('..\graustufe.bmp');
  Texture.Activate;                         //standard stage 0
  Texture.ColorStageBlendTextureOnly;       //use texture color only, ignore vertex diffuse color
  Texture.AlphaStageBlendDiffuseOnly;       //use alpha values of vertices only (set by monoalpha), ignore texture alpha values
  //second rendering (source)
  AlphaBlending.Activate;                   //activate alpha blending (alpha blending = how to mix this source with previous "destination")
  MultiMap.Draw;                            //second rendering
  AlphaBlending.Deactivate;                 //deactivate alpha blending
  Texture.Deactivate;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
