unit PittiToMapMain;

interface

uses
  SysUtils, Forms, Windows,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Map: TxorDXMap;
    Tex: TxorDXTexture;
    Mouse: TxorDXMouse;
    Keyboard: TxorDXKeyboard;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Graphics;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
var
  u, v: Integer;
begin
  Scene  := TxorDXScene.Create(Handle);
  //Scene  := TxorDXScene.CreateFullScreen(Handle);   //fullscreen, close with Alt-F4

  Camera := TxorDXCamera.Create(Scene);
  Camera.Position := GetVector(0.6, 0.6, -1); //do a change
  Camera.FixTransformation;                   //fix that change
  Camera.Activate;                            //activate changes

  Mouse    := TxorDXMouse.Create(Scene);
  Mouse.InvertX := True;
  Keyboard := TxorDXKeyboard.Create(Scene);

  //30x30 Map
  //using vertices which are able to handle textures: type TxorDXVertexTexture
  Map := TxorDXMap.CreateParam(Scene, 30, 30, xorDXVertexSizeTexture, xorDXVertexFlagTexture);
  for u := 0 to Map.MapWidth - 1 do
    for v := 0 to Map.MapHeight - 1 do
      with PxorDXVertexTexture(Map.Vertex[u,v])^ do
        begin
        Position := GetVector(u / (Map.MapWidth - 1) - 0.5, Sqr(Sqr(Sqr(Random))) / 10, v / (Map.MapHeight - 1) - 0.5);
        TexCoordinates := GetCoord(u / (Map.MapWidth - 1), v / (Map.MapHeight - 1));  //maps texture coordinates to the map coordinates
        end;
  Map.InitNormalVectors; //get average normalvector of each point
  Map.MonoChrom(clWhite); //vertex color

  Tex := TxorDXTexture.Create(Scene);
  Tex.LoadFromFile('..\PittiToMap\pitti.jpg');
  Tex.ColorStageBlendTextureOnly; //dont mix the vertex color and texture color, use texture color only
  Tex.Activate;

  Scene.CreateBuffers;
  Scene.StartLooping(Draw);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  if Assigned(Scene) then Scene.StopLooping;
  //free in inverted order of creation
  if Assigned(Tex) then FreeAndNil(Tex);
  if Assigned(Map) then FreeAndNil(Map);
  if Assigned(Keyboard) then FreeAndNil(Keyboard);
  if Assigned(Mouse) then FreeAndNil(Mouse);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;

  if Mouse.Get then //read mouse buffer
    with Camera do
      begin
      RotateHoriz(Mouse.DX/100, xorDXCamHuman);
      RotateVert(Mouse.DY/100, xorDXCamHuman);
      if Mouse.ClickedLeft then UndoTransformation; //undo not fixed transformations
      end;

  if Keyboard.Get and Keyboard.KeyPressedAny then //read keyboard buffer
    with Camera do
      begin
      //use windows "virtual keycodes"
      if Keyboard.KeyPressed(VK_ESCAPE) then Application.Terminate;
      if Keyboard.KeyPressed(VK_LEFT) then MoveHoriz(-0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_RIGHT) then MoveHoriz(0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_UP) then MoveForward(0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_DOWN) then MoveForward(-0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_PRIOR) then MoveVert(0.03, xorDXCamId);
      if Keyboard.KeyPressed(VK_NEXT) then MoveVert(-0.03, xorDXCamId);
      end;

  Camera.Activate; //setting changes

  Map.Insert;
  Map.Draw;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;
  InitRotationPerSecond;
  Randomize;

end.
