unit TextureMain;

interface

uses
  SysUtils, Forms,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormPaint(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    MultiMap: TxorDXMultiMap;
    Texture0: TxorDXTexture;
    Texture1: TxorDXTexture;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Graphics;

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  //creation
  Scene    := TxorDXScene.Create(Handle);
  Camera   := TxorDXCamera.Create(Scene);
  MultiMap := TxorDXMultiMap.CreateParam(Scene, 50, 50, xorDXVertexSizeTexture, xorDXVertexFlagTexture);
  Texture0 := TxorDXTexture.Create(Scene);
  Texture1 := TxorDXTexture.Create(Scene);
  Scene.CreateBuffers;
  //load and put textures to stages
  Texture0.LoadFromFile('..\Texture\agypt.bmp');
  Texture1.LoadFromFile('..\Texture\redstone.bmp');
  Texture0.Stage := 0;
  Texture1.Stage := 1;
  Texture0.Activate;
  Texture1.Activate;
  //settings
  MultiMap.MapParallelogram(GetVector(0,2,0), GetVector(0,0,0), GetVector(2,0,0));
  Camera.MoveForward(-8, xorDXCamID);
  Camera.Ratio := Width/Height;
  Camera.Activate;
  Scene.OnDraw := Draw; //set draw routine
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  Texture1.Deactivate;
  Texture0.Deactivate;
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(Texture1) then FreeAndNil(Texture1);
  if Assigned(Texture0) then FreeAndNil(Texture0);
  if Assigned(MultiMap) then FreeAndNil(MultiMap);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//onPaint event of TForm1
procedure TForm1.FormPaint(Sender: TObject);
begin
  Scene.Draw;
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;
  MultiMap.Insert;
  //show rect with diffuse color
  MultiMap.Move(-2, 2, 0);
  Texture0.ColorStageBlendDisable;
  Texture1.ColorStageBlendDisable;
  MultiMap.Draw;
  //show rect with texture0 only
  MultiMap.Move(2, 0, 0);
  Texture0.ColorStageBlendTextureOnly;
  Texture1.ColorStageBlendDisable;
  MultiMap.Draw;
  //show rect with texture0 * diffuse color
  MultiMap.Move(-2, -2, 0);
  Texture0.ColorStageBlendModulate;
  Texture1.ColorStageBlendDisable;
  MultiMap.Draw;
  //show rect with texture1 only
  MultiMap.Move(2, 0, 0);
  Texture0.ColorStageBlendModulate;
  Texture1.ColorStageBlendTextureOnly;
  MultiMap.Draw;
  //show rect with texture0 * diffuse color + texture1
  MultiMap.Move(-1, -2, 0);
  Texture0.ColorStageBlendModulate;
  Texture1.ColorStageBlendAdd;
  MultiMap.Draw;
  MultiMap.UndoTransformation;
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
