unit VertexAndIndexMain;

interface

uses
  SysUtils, Forms,
  xorDX8;

type
  TForm1 = class(TForm)
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
    Scene: TxorDXScene;
    Camera: TxorDXCamera;
    Texture: TxorDXTexture;
    PolygonClassic: TxorDXPolygon;
    PolygonTexture: TxorDXPolygon;
    PolygonUserDef: TxorDXPolygon;
    IndexedPolygon: TxorDXIndexedPolygon;
    procedure Draw(Sender: TObject);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

uses Graphics, DirectXGraphics;

type
  //userdefined vertex format;
  PVertexUserDef = ^TVertexUserDef;
  TVertexUserDef = packed record
                   Position: TxorDXVector;
                   TexCoordinates0: TxorDXCoord;
                   TexCoordinates1: TxorDXCoord;
                   end;
const
  VertexFlagUserDef = D3DFVF_XYZ or D3DFVF_TEX2; //D3D-const in unit DirectXGraphics
  VertexSizeUserDef = SizeOf(TVertexUserDef);

{ TForm1 }

procedure TForm1.FormCreate(Sender: TObject);
begin
  //creation
  Scene  := TxorDXScene.Create(Handle);
  Camera := TxorDXCamera.Create(Scene);
  Texture := TxorDXTexture.Create(Scene);
  PolygonClassic := TxorDXPolygon.Create(Scene, 4);
  PolygonTexture := TxorDXPolygon.CreateParam(Scene, 4, xorDXVertexSizeTexture, xorDXVertexFlagTexture);
  PolygonUserDef := TxorDXPolygon.CreateParam(Scene, 4, VertexSizeUserDef, VertexFlagUserDef);
  IndexedPolygon := TxorDXIndexedPolygon.Create(Scene, 4, 6);
  Scene.CreateBuffers;
  //set vertices
  //default drawmode = xorDXPrimTriangleStrip (triangles: 012 and 123)
  //normals not need, set to 0,0,0
  with PolygonClassic do
    begin
    PxorDXVertexClassic(Vertex[0])^ := GetVertexClassic(2,0,0,  0,0,0, clBlack, 0);
    PxorDXVertexClassic(Vertex[1])^ := GetVertexClassic(2,1,0,  0,0,0, clBlue, 0);
    PxorDXVertexClassic(Vertex[2])^ := GetVertexClassic(3,0,0,  0,0,0, clRed, 0);
    PxorDXVertexClassic(Vertex[3])^ := GetVertexClassic(3,0,1,  0,0,0, clGreen, 0);
    end;
  with PolygonTexture do
    begin
    PxorDXVertexTexture(Vertex[0])^ := GetVertexTexture(-1,0,0,  0,0,0, 0,1, clBlack, 0);
    PxorDXVertexTexture(Vertex[1])^ := GetVertexTexture(-1,1,0,  0,0,0, 0,0, clBlue, 0);
    PxorDXVertexTexture(Vertex[2])^ := GetVertexTexture( 0,0,0,  0,0,0, 1,1, clRed, 0);
    PxorDXVertexTexture(Vertex[3])^ := GetVertexTexture( 0,0,1,  0,0,0, 1,0, clGreen, 0);
    end;
  with PolygonUserDef do
    begin
    with PVertexUserDef(Vertex[0])^ do
      begin
      Position := GetVector(-4,0,0);
      TexCoordinates0 := GetCoord(0,1);
      TexCoordinates1 := GetCoord(0.0, 0.5)
      end;
    with PVertexUserDef(Vertex[1])^ do
      begin
      Position := GetVector(-4,1,0);
      TexCoordinates0 := GetCoord(0,0);
      TexCoordinates1 := GetCoord(0.0, 0.0)
      end;
    with PVertexUserDef(Vertex[2])^ do
      begin
      Position := GetVector(-3,0,0);
      TexCoordinates0 := GetCoord(1,1);
      TexCoordinates1 := GetCoord(0.5, 0.5)
      end;
    with PVertexUserDef(Vertex[3])^ do
      begin
      Position := GetVector(-3,0,1);
      TexCoordinates0 := GetCoord(1,0);
      TexCoordinates1 := GetCoord(0.5, 0.0)
      end;
    end;
  with IndexedPolygon do
    begin
    DrawMode := xorDXDrawTriangleStrip;
    PxorDXVertexClassic(Vertex[0])^ := GetVertexClassic(-1,0,0,  0,0,0, clBlack, 0);
    PxorDXVertexClassic(Vertex[1])^ := GetVertexClassic(-1,1,0,  0,0,0, clBlue, 0);
    PxorDXVertexClassic(Vertex[2])^ := GetVertexClassic( 0,0,0,  0,0,0, clRed, 0);
    PxorDXVertexClassic(Vertex[3])^ := GetVertexClassic( 0,0,1,  0,0,0, clGreen, 0);
    Idx[0] := 0;
    Idx[1] := 1;
    Idx[2] := 2;  //triangle 0 = 0,1,2
    Idx[3] := 3;  //triangle 1 = 1,2,3
    Idx[4] := 0;  //triangle 2 = 2,3,0
    Idx[5] := 1;  //triangle 3 = 3,0,1
    end;
  Texture.LoadFromFile('../dali.bmp');
  Texture.Activate;
  Camera.OrthogonalProjection := True;
  Camera.Activate;
  Scene.StartLooping(Draw); //loop drawing routine
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  //free in inverted order of creation
  if Assigned(Scene) then Scene.StopLooping;
  if Assigned(IndexedPolygon) then FreeAndNil(IndexedPolygon);
  if Assigned(PolygonUserDef) then FreeAndNil(PolygonUserDef);
  if Assigned(PolygonTexture) then FreeAndNil(PolygonTexture);
  if Assigned(PolygonClassic) then FreeAndNil(PolygonClassic);
  if Assigned(Texture) then FreeAndNil(Texture);
  if Assigned(Camera) then FreeAndNil(Camera);
  if Assigned(Scene) then FreeAndNil(Scene);
end;

//repeated drawing routine
procedure TForm1.Draw(Sender: TObject);
begin
  Scene.Clear;
  PolygonClassic.Insert;
  PolygonClassic.RotateX(0.01);
  PolygonClassic.Draw;
  Texture.ColorStageBlendTextureOnly;
  PolygonTexture.Insert;
  PolygonTexture.RotateX(0.01);
  PolygonTexture.Draw;
  Texture.SetTextureCoordinates(xorDXTexCoordIndex1);
  PolygonUserDef.Insert;
  PolygonUserDef.RotateX(0.01);
  PolygonUserDef.Draw;
  Texture.SetTextureCoordinates(xorDXTexCoordIndex0);
  Texture.ColorStageBlendDisable;
  IndexedPolygon.Insert;
  IndexedPolygon.RotateX(0.01);
  IndexedPolygon.FixTransformation; //rotation is fixed
  IndexedPolygon.Move(0, 3, 0);
  IndexedPolygon.Draw;
  IndexedPolygon.UndoTransformation; //undo move, but rotation is hold
end;

initialization

  xorDXThrowAllOtherExceptions := True;
  xorDXThrowCameraTransformationExceptions := False;
  xorDXThrowIndexSizeException := True;
  xorDXThrowCapsException := True;

end.
